@@ -26,28 +26,37 @@ import (
 	k8syaml "k8s.io/apimachinery/pkg/util/yaml"
 )
 
+const (
+	DefaultRetentionValue = "15d"
+)
+
 type Config struct {
 	Images      *Images               `json:"-"`
 	RemoteWrite bool                  `json:"-"`
 	Platform    configv1.PlatformType `json:"-"`
 
-	PrometheusOperatorConfig             *PrometheusOperatorConfig `json:"prometheusOperator"`
+	ClusterMonitoringConfiguration *ClusterMonitoringConfiguration `json:"-"`
+	UserWorkloadConfiguration      *UserWorkloadConfiguration      `json:"-"`
+}
+
+type ClusterMonitoringConfiguration struct {
+	PrometheusOperatorConfig *PrometheusOperatorConfig    `json:"prometheusOperator"`
+	PrometheusK8sConfig      *PrometheusK8sConfig         `json:"prometheusK8s"`
+	AlertmanagerMainConfig   *AlertmanagerMainConfig      `json:"alertmanagerMain"`
+	KubeStateMetricsConfig   *KubeStateMetricsConfig      `json:"kubeStateMetrics"`
+	OpenShiftMetricsConfig   *OpenShiftStateMetricsConfig `json:"openshiftStateMetrics"`
+	GrafanaConfig            *GrafanaConfig               `json:"grafana"`
+	EtcdConfig               *EtcdConfig                  `json:"-"`
+	HTTPConfig               *HTTPConfig                  `json:"http"`
+	TelemeterClientConfig    *TelemeterClientConfig       `json:"telemeterClient"`
+	K8sPrometheusAdapter     *K8sPrometheusAdapter        `json:"k8sPrometheusAdapter"`
+	ThanosQuerierConfig      *ThanosQuerierConfig         `json:"thanosQuerier"`
+	UserWorkloadEnabled      *bool                        `json:"enableUserWorkload"`
+	// TODO: Remove in 4.7 release.
+	PrometheusUserWorkloadConfig         *PrometheusK8sConfig      `json:"prometheusUserWorkload"`
 	PrometheusOperatorUserWorkloadConfig *PrometheusOperatorConfig `json:"prometheusOperatorUserWorkload"`
-
-	PrometheusK8sConfig          *PrometheusK8sConfig `json:"prometheusK8s"`
-	PrometheusUserWorkloadConfig *PrometheusK8sConfig `json:"prometheusUserWorkload"`
-
-	AlertmanagerMainConfig *AlertmanagerMainConfig      `json:"alertmanagerMain"`
-	KubeStateMetricsConfig *KubeStateMetricsConfig      `json:"kubeStateMetrics"`
-	OpenShiftMetricsConfig *OpenShiftStateMetricsConfig `json:"openshiftStateMetrics"`
-	GrafanaConfig          *GrafanaConfig               `json:"grafana"`
-	EtcdConfig             *EtcdConfig                  `json:"-"`
-	HTTPConfig             *HTTPConfig                  `json:"http"`
-	TelemeterClientConfig  *TelemeterClientConfig       `json:"telemeterClient"`
-	K8sPrometheusAdapter   *K8sPrometheusAdapter        `json:"k8sPrometheusAdapter"`
-	UserWorkloadConfig     *UserWorkloadConfig          `json:"techPreviewUserWorkload"`
-	ThanosRulerConfig      *ThanosRulerConfig           `json:"thanosRuler"`
-	ThanosQuerierConfig    *ThanosQuerierConfig         `json:"thanosQuerier"`
+	ThanosRulerConfig                    *ThanosRulerConfig        `json:"thanosRuler"`
+	UserWorkloadConfig                   *UserWorkloadConfig       `json:"techPreviewUserWorkload"`
 }
 
 type Images struct {
@@ -75,18 +84,19 @@ type HTTPConfig struct {
 }
 
 type PrometheusOperatorConfig struct {
+	LogLevel     string            `json:"logLevel"`
 	NodeSelector map[string]string `json:"nodeSelector"`
 	Tolerations  []v1.Toleration   `json:"tolerations"`
 }
 
 type PrometheusK8sConfig struct {
+	LogLevel            string                               `json:"logLevel"`
 	Retention           string                               `json:"retention"`
 	NodeSelector        map[string]string                    `json:"nodeSelector"`
 	Tolerations         []v1.Toleration                      `json:"tolerations"`
 	Resources           *v1.ResourceRequirements             `json:"resources"`
 	ExternalLabels      map[string]string                    `json:"externalLabels"`
 	VolumeClaimTemplate *monv1.EmbeddedPersistentVolumeClaim `json:"volumeClaimTemplate"`
-	Hostport            string                               `json:"hostport"`
 	RemoteWrite         []monv1.RemoteWriteSpec              `json:"remoteWrite"`
 	TelemetryMatches    []string                             `json:"-"`
 }
@@ -96,10 +106,10 @@ type AlertmanagerMainConfig struct {
 	Tolerations         []v1.Toleration                      `json:"tolerations"`
 	Resources           *v1.ResourceRequirements             `json:"resources"`
 	VolumeClaimTemplate *monv1.EmbeddedPersistentVolumeClaim `json:"volumeClaimTemplate"`
-	Hostport            string                               `json:"hostport"`
 }
 
 type ThanosRulerConfig struct {
+	LogLevel            string                               `json:"logLevel"`
 	NodeSelector        map[string]string                    `json:"nodeSelector"`
 	Tolerations         []v1.Toleration                      `json:"tolerations"`
 	Resources           *v1.ResourceRequirements             `json:"resources"`
@@ -115,7 +125,6 @@ type ThanosQuerierConfig struct {
 type GrafanaConfig struct {
 	NodeSelector map[string]string `json:"nodeSelector"`
 	Tolerations  []v1.Toleration   `json:"tolerations"`
-	Hostport     string            `json:"hostport"`
 }
 
 type KubeStateMetricsConfig struct {
@@ -150,15 +159,6 @@ type UserWorkloadConfig struct {
 	Enabled *bool `json:"enabled"`
 }
 
-// IsEnabled returns the underlying value of the `Enabled` boolean pointer.
-// It defaults to false if the pointer is nil.
-func (c *UserWorkloadConfig) IsEnabled() bool {
-	if c.Enabled == nil {
-		return false
-	}
-	return *c.Enabled
-}
-
 type TelemeterClientConfig struct {
 	ClusterID          string            `json:"clusterID"`
 	Enabled            *bool             `json:"enabled"`
@@ -184,14 +184,15 @@ func (cfg *TelemeterClientConfig) IsEnabled() bool {
 
 func NewConfig(content io.Reader) (*Config, error) {
 	c := Config{}
-
-	err := k8syaml.NewYAMLOrJSONDecoder(content, 100).Decode(&c)
+	cmc := ClusterMonitoringConfiguration{}
+	err := k8syaml.NewYAMLOrJSONDecoder(content, 4096).Decode(&cmc)
 	if err != nil {
 		return nil, err
 	}
-
+	c.ClusterMonitoringConfiguration = &cmc
 	res := &c
 	res.applyDefaults()
+	c.UserWorkloadConfiguration = NewDefaultUserWorkloadMonitoringConfig()
 
 	return res, nil
 }
@@ -200,58 +201,65 @@ func (c *Config) applyDefaults() {
 	if c.Images == nil {
 		c.Images = &Images{}
 	}
-	if c.PrometheusOperatorConfig == nil {
-		c.PrometheusOperatorConfig = &PrometheusOperatorConfig{}
+	if c.ClusterMonitoringConfiguration == nil {
+		c.ClusterMonitoringConfiguration = &ClusterMonitoringConfiguration{}
 	}
-	if c.PrometheusOperatorUserWorkloadConfig == nil {
-		c.PrometheusOperatorUserWorkloadConfig = &PrometheusOperatorConfig{}
+	if c.ClusterMonitoringConfiguration.PrometheusOperatorConfig == nil {
+		c.ClusterMonitoringConfiguration.PrometheusOperatorConfig = &PrometheusOperatorConfig{}
 	}
-	if c.PrometheusK8sConfig == nil {
-		c.PrometheusK8sConfig = &PrometheusK8sConfig{}
+	if c.ClusterMonitoringConfiguration.PrometheusOperatorUserWorkloadConfig == nil {
+		c.ClusterMonitoringConfiguration.PrometheusOperatorUserWorkloadConfig = &PrometheusOperatorConfig{}
 	}
-	if c.PrometheusK8sConfig.Retention == "" {
-		c.PrometheusK8sConfig.Retention = "15d"
+	if c.ClusterMonitoringConfiguration.PrometheusK8sConfig == nil {
+		c.ClusterMonitoringConfiguration.PrometheusK8sConfig = &PrometheusK8sConfig{}
 	}
-	if c.PrometheusUserWorkloadConfig == nil {
-		c.PrometheusUserWorkloadConfig = &PrometheusK8sConfig{}
+	if c.ClusterMonitoringConfiguration.PrometheusK8sConfig.Retention == "" {
+		c.ClusterMonitoringConfiguration.PrometheusK8sConfig.Retention = DefaultRetentionValue
 	}
-	if c.PrometheusUserWorkloadConfig.Retention == "" {
-		c.PrometheusUserWorkloadConfig.Retention = "15d"
+	if c.ClusterMonitoringConfiguration.PrometheusUserWorkloadConfig == nil {
+		c.ClusterMonitoringConfiguration.PrometheusUserWorkloadConfig = &PrometheusK8sConfig{}
 	}
-	if c.AlertmanagerMainConfig == nil {
-		c.AlertmanagerMainConfig = &AlertmanagerMainConfig{}
+	if c.ClusterMonitoringConfiguration.PrometheusUserWorkloadConfig.Retention == "" {
+		c.ClusterMonitoringConfiguration.PrometheusUserWorkloadConfig.Retention = DefaultRetentionValue
 	}
-	if c.ThanosRulerConfig == nil {
-		c.ThanosRulerConfig = &ThanosRulerConfig{}
+	if c.ClusterMonitoringConfiguration.AlertmanagerMainConfig == nil {
+		c.ClusterMonitoringConfiguration.AlertmanagerMainConfig = &AlertmanagerMainConfig{}
 	}
-	if c.ThanosQuerierConfig == nil {
-		c.ThanosQuerierConfig = &ThanosQuerierConfig{}
+	if c.ClusterMonitoringConfiguration.UserWorkloadEnabled == nil {
+		disable := false
+		c.ClusterMonitoringConfiguration.UserWorkloadEnabled = &disable
 	}
-	if c.GrafanaConfig == nil {
-		c.GrafanaConfig = &GrafanaConfig{}
+	if c.ClusterMonitoringConfiguration.ThanosRulerConfig == nil {
+		c.ClusterMonitoringConfiguration.ThanosRulerConfig = &ThanosRulerConfig{}
 	}
-	if c.KubeStateMetricsConfig == nil {
-		c.KubeStateMetricsConfig = &KubeStateMetricsConfig{}
+	if c.ClusterMonitoringConfiguration.ThanosQuerierConfig == nil {
+		c.ClusterMonitoringConfiguration.ThanosQuerierConfig = &ThanosQuerierConfig{}
 	}
-	if c.OpenShiftMetricsConfig == nil {
-		c.OpenShiftMetricsConfig = &OpenShiftStateMetricsConfig{}
+	if c.ClusterMonitoringConfiguration.GrafanaConfig == nil {
+		c.ClusterMonitoringConfiguration.GrafanaConfig = &GrafanaConfig{}
 	}
-	if c.HTTPConfig == nil {
-		c.HTTPConfig = &HTTPConfig{}
+	if c.ClusterMonitoringConfiguration.KubeStateMetricsConfig == nil {
+		c.ClusterMonitoringConfiguration.KubeStateMetricsConfig = &KubeStateMetricsConfig{}
 	}
-	if c.TelemeterClientConfig == nil {
-		c.TelemeterClientConfig = &TelemeterClientConfig{
+	if c.ClusterMonitoringConfiguration.OpenShiftMetricsConfig == nil {
+		c.ClusterMonitoringConfiguration.OpenShiftMetricsConfig = &OpenShiftStateMetricsConfig{}
+	}
+	if c.ClusterMonitoringConfiguration.HTTPConfig == nil {
+		c.ClusterMonitoringConfiguration.HTTPConfig = &HTTPConfig{}
+	}
+	if c.ClusterMonitoringConfiguration.TelemeterClientConfig == nil {
+		c.ClusterMonitoringConfiguration.TelemeterClientConfig = &TelemeterClientConfig{
 			TelemeterServerURL: "https://infogw.api.openshift.com/",
 		}
 	}
-	if c.K8sPrometheusAdapter == nil {
-		c.K8sPrometheusAdapter = &K8sPrometheusAdapter{}
+	if c.ClusterMonitoringConfiguration.K8sPrometheusAdapter == nil {
+		c.ClusterMonitoringConfiguration.K8sPrometheusAdapter = &K8sPrometheusAdapter{}
 	}
-	if c.EtcdConfig == nil {
-		c.EtcdConfig = &EtcdConfig{}
+	if c.ClusterMonitoringConfiguration.EtcdConfig == nil {
+		c.ClusterMonitoringConfiguration.EtcdConfig = &EtcdConfig{}
 	}
-	if c.UserWorkloadConfig == nil {
-		c.UserWorkloadConfig = &UserWorkloadConfig{}
+	if c.ClusterMonitoringConfiguration.UserWorkloadConfig == nil {
+		c.ClusterMonitoringConfiguration.UserWorkloadConfig = &UserWorkloadConfig{}
 	}
 }
 
@@ -274,18 +282,18 @@ func (c *Config) SetImages(images map[string]string) {
 }
 
 func (c *Config) SetTelemetryMatches(matches []string) {
-	c.PrometheusK8sConfig.TelemetryMatches = matches
+	c.ClusterMonitoringConfiguration.PrometheusK8sConfig.TelemetryMatches = matches
 }
 
 func (c *Config) SetRemoteWrite(rw bool) {
 	c.RemoteWrite = rw
-	if c.RemoteWrite && c.TelemeterClientConfig.TelemeterServerURL == "https://infogw.api.openshift.com/" {
-		c.TelemeterClientConfig.TelemeterServerURL = "https://infogw.api.openshift.com/metrics/v1/receive"
+	if c.RemoteWrite && c.ClusterMonitoringConfiguration.TelemeterClientConfig.TelemeterServerURL == "https://infogw.api.openshift.com/" {
+		c.ClusterMonitoringConfiguration.TelemeterClientConfig.TelemeterServerURL = "https://infogw.api.openshift.com/metrics/v1/receive"
 	}
 }
 
 func (c *Config) LoadClusterID(load func() (*configv1.ClusterVersion, error)) error {
-	if c.TelemeterClientConfig.ClusterID != "" {
+	if c.ClusterMonitoringConfiguration.TelemeterClientConfig.ClusterID != "" {
 		return nil
 	}
 
@@ -294,12 +302,12 @@ func (c *Config) LoadClusterID(load func() (*configv1.ClusterVersion, error)) er
 		return fmt.Errorf("error loading cluster version: %v", err)
 	}
 
-	c.TelemeterClientConfig.ClusterID = string(cv.Spec.ClusterID)
+	c.ClusterMonitoringConfiguration.TelemeterClientConfig.ClusterID = string(cv.Spec.ClusterID)
 	return nil
 }
 
 func (c *Config) LoadToken(load func() (*v1.Secret, error)) error {
-	if c.TelemeterClientConfig.Token != "" {
+	if c.ClusterMonitoringConfiguration.TelemeterClientConfig.Token != "" {
 		return nil
 	}
 
@@ -324,12 +332,12 @@ func (c *Config) LoadToken(load func() (*v1.Secret, error)) error {
 		return fmt.Errorf("unmarshaling pull secret failed: %v", err)
 	}
 
-	c.TelemeterClientConfig.Token = ps.Auths.COC.Auth
+	c.ClusterMonitoringConfiguration.TelemeterClientConfig.Token = ps.Auths.COC.Auth
 	return nil
 }
 
 func (c *Config) LoadProxy(load func() (*configv1.Proxy, error)) error {
-	if c.HTTPConfig.HTTPProxy != "" || c.HTTPConfig.HTTPSProxy != "" || c.HTTPConfig.NoProxy != "" {
+	if c.ClusterMonitoringConfiguration.HTTPConfig.HTTPProxy != "" || c.ClusterMonitoringConfiguration.HTTPConfig.HTTPSProxy != "" || c.ClusterMonitoringConfiguration.HTTPConfig.NoProxy != "" {
 		return nil
 	}
 
@@ -338,9 +346,9 @@ func (c *Config) LoadProxy(load func() (*configv1.Proxy, error)) error {
 		return fmt.Errorf("error loading proxy: %v", err)
 	}
 
-	c.HTTPConfig.HTTPProxy = p.Status.HTTPProxy
-	c.HTTPConfig.HTTPSProxy = p.Status.HTTPSProxy
-	c.HTTPConfig.NoProxy = p.Status.NoProxy
+	c.ClusterMonitoringConfiguration.HTTPConfig.HTTPProxy = p.Status.HTTPProxy
+	c.ClusterMonitoringConfiguration.HTTPConfig.HTTPSProxy = p.Status.HTTPSProxy
+	c.ClusterMonitoringConfiguration.HTTPConfig.NoProxy = p.Status.NoProxy
 
 	return nil
 }
@@ -364,6 +372,79 @@ func NewConfigFromString(content string) (*Config, error) {
 
 func NewDefaultConfig() *Config {
 	c := &Config{}
+	cmc := ClusterMonitoringConfiguration{}
+	c.ClusterMonitoringConfiguration = &cmc
+	c.UserWorkloadConfiguration = NewDefaultUserWorkloadMonitoringConfig()
 	c.applyDefaults()
 	return c
 }
+
+type UserWorkloadConfiguration struct {
+	PrometheusOperator *PrometheusOperatorConfig   `json:"prometheusOperator"`
+	Prometheus         *PrometheusRestrictedConfig `json:"prometheus"`
+	ThanosRuler        *ThanosRulerConfig          `json:"thanosRuler"`
+}
+
+type PrometheusRestrictedConfig struct {
+	LogLevel            string                               `json:"logLevel"`
+	Retention           string                               `json:"retention"`
+	NodeSelector        map[string]string                    `json:"nodeSelector"`
+	Tolerations         []v1.Toleration                      `json:"tolerations"`
+	Resources           *v1.ResourceRequirements             `json:"resources"`
+	ExternalLabels      map[string]string                    `json:"externalLabels"`
+	VolumeClaimTemplate *monv1.EmbeddedPersistentVolumeClaim `json:"volumeClaimTemplate"`
+	RemoteWrite         []monv1.RemoteWriteSpec              `json:"remoteWrite"`
+	EnforcedSampleLimit *uint64                              `json:"enforcedSampleLimit"`
+}
+
+func (u *UserWorkloadConfiguration) applyDefaults() {
+	if u.PrometheusOperator == nil {
+		u.PrometheusOperator = &PrometheusOperatorConfig{}
+	}
+	if u.Prometheus == nil {
+		u.Prometheus = &PrometheusRestrictedConfig{}
+	}
+	if u.ThanosRuler == nil {
+		u.ThanosRuler = &ThanosRulerConfig{}
+	}
+}
+
+func NewUserConfigFromString(content string) (*UserWorkloadConfiguration, error) {
+	if content == "" {
+		return NewDefaultUserWorkloadMonitoringConfig(), nil
+	}
+	u := &UserWorkloadConfiguration{}
+	err := k8syaml.NewYAMLOrJSONDecoder(bytes.NewBuffer([]byte(content)), 100).Decode(&u)
+	if err != nil {
+		return nil, err
+	}
+
+	u.applyDefaults()
+
+	return u, nil
+}
+
+func NewDefaultUserWorkloadMonitoringConfig() *UserWorkloadConfiguration {
+	u := &UserWorkloadConfiguration{}
+	u.applyDefaults()
+	return u
+}
+
+// IsUserWorkloadEnabled checks if user workload monitoring is
+// enabled on old or new configuration.
+func (c *Config) IsUserWorkloadEnabled() bool {
+	if *c.ClusterMonitoringConfiguration.UserWorkloadEnabled == true {
+		return true
+	}
+
+	return c.ClusterMonitoringConfiguration.UserWorkloadConfig.isEnabled()
+}
+
+// isEnabled returns the underlying value of the `Enabled` boolean pointer.
+// It defaults to false if the pointer is nil.
+func (c *UserWorkloadConfig) isEnabled() bool {
+	if c.Enabled == nil {
+		return false
+	}
+	return *c.Enabled
+}
